// icat.spat tools by Ivica Ico Bukvic (c) 2014 <ico@vt.edu>
// distributed under GNU GPL license 3 or greater
// for more info on the license visit http://www.gnu.org/copyleft/gpl.html

import com.cycling74.max.*;

public class icat_spat_horizontal extends MaxObject {

	
	
  private float[] speakers;
  private int[] speaker_id;
  private float amp = 1;
  private float gain = 1;
  private float range = 360;
  private int mask = 0;
  private int length = 0;
  private String context = "null";

  
  
  public icat_spat_horizontal(String name)
  {
	declareIO(1,1);
	setInletAssist(new String[] { "float list or setup" });
	setOutletAssist(new String[] { "float list: send, amplitude" });
	
	context = name;
		
  }

  
  
  //main calculation of the amplitude ratios
  public void inlet(float val)
  {
  	if (length == 0)
  	{
  		error("icat_spat_horizontal: no speaker setup provided\n");
  		return;
  	}
  			
  	int left = -1;
  	int right = -1;
  	float l_amp = 0;
  	float r_amp = 0;
  	float distance = 0;
  	int i = 0;
 
  	if (val < 0)
  		while (val < 0)
  			val += range;
  	else if (val > range)
  		while (val > range)
  			val -= range;

  	for (i = 0; i < length; i++)
  	{
  		if (speakers[i] <= val)
  			left = i;
  		else
  		{
  			right = i;
  			break;
  		}
  	}
  	if (left == -1)
  		left = length - 1;
  	if (right == -1)
  		right = 0;		
  	
  	//post("left=" + left + " right=" + right + " length=" + length + "\n");
  	
  	if (right != 0)
  		distance = speakers[right] - speakers[left];
  	else
  		distance = range + speakers[right] - speakers[left];
  	
  	// we add range (360) in cases where val is e.g. 0 and is in between two speakers, neither of which are at 0
  	if (val < speakers[left])
  		val += range;
  	l_amp = (float)Math.sqrt(1 - (val - speakers[left])/distance);
  	r_amp = (float)Math.sqrt((val - speakers[left])/distance);
  	
  	//post("distance=" + distance + " l_amp=" + l_amp + " r_amp=" + r_amp + "\n");
  	//post("mask=" + mask + "\n");
  	
  	for (i = 0; i < speakers.length; i++)
  	{
  		if (i == left)
  		{
  			if (mask == 0 || (mask == 1 && l_amp * amp * gain != 0))
  				outlet(0, context + speaker_id[i], l_amp * amp * gain);
  		}
  		else if (i == right)
  		{
  			if (mask == 0 || (mask == 1 && r_amp * amp * gain != 0))
  				outlet(0, context + speaker_id[i], r_amp * amp * gain);
  		}
  		else
  		{
  			if (mask == 0)
  				outlet(0, context + speaker_id[i], 0);
  		}
  	}	
  }
  
  
  
  //redirect for integer values to the main calculation function above
  public void inlet(int val)
  {
	inlet((float)val);
  }
  
  
  
  //set up speaker array locations on a horizontal plane
  public void setup(float[] arguments)
  {
	  if (arguments.length > 0)
	  {
		  int i = 0;
		  int j = 0;
		  speakers = null;
		  speaker_id = null;
		  speakers = new float[arguments.length / 2];
		  speaker_id = new int[arguments.length / 2];
		  for (i = 0; i < arguments.length; i++)
		  {
			  //post("loop i=" + i + "\n");
			  if (i % 2 == 0)
			  {
				  // even entries are speaker id (which channel are we mapping onto)
				  if (arguments[i] < 1 || arguments[i] > 512)
					  error("icat_spat_horizontal: speaker ids should be inside the range of 1-512\n");
				  else
					  speaker_id[j] = (int)arguments[i];
				  //post("speaker_id=" + speaker_id[j] + "\n");
			  }
			  else
			  {
				  //post(j + " " + arguments[i] + " " + speaker_id[j] + "\n");
				  // odd entries are speaker angles
				  if (arguments[i] < 0 || arguments[i] >= range)
					  error("icat_spat_horizontali: speaker array should have angles between 0 and " + range + "\n");
				  else if (j > 0 && arguments[i] < speakers[j])
				  {
					  error("icat_spat_horizontal: speaker array should be provided sequentially in ascending order from 0 to " + range + "\n");
				  }
				  else speakers[j] = arguments[i];
				  //post("speakers[j]=" + speakers[j] + "\n");
				  j++;
			  }
		  }
		  length = (int) arguments.length / 2;
	  }
  }

  
  
  //list of three values -- default implementation when using with ico.sound~ and icat_spat_horizontal script
  public void list(Atom[] arguments)
  {
	if (arguments.length == 4)
	{
		int i = 0;
		for (i = 3; i >= 0; i--)
		{
			if (arguments[i].isFloat())
			{
				switch (i)
				{
					case 0:
						inlet(arguments[i].getFloat());
						break;
					case 1:
						amp = arguments[i].getFloat();
						break;
					case 2:
						gain = arguments[i].getFloat();
						break;
					case 3:
						mask = (int)arguments[i].getFloat();
						break;
					default:
						break;
				}
					
			}
			else if (arguments[i].isInt())
			{
				switch (i)
				{
					case 0:
						inlet(arguments[i].getInt());
						break;
					case 1:
						amp = arguments[i].getInt();
						break;
					case 2:
						gain = arguments[i].getInt();
						break;
					case 3:
						mask = arguments[i].getInt();
						break;
					default:
						break;
				}
			}
			else 
			{
				error("icat_spat_horizontal: invalid values (should be int/float list)\n");
				break;
			}
		}
	}
  }
  
  
  
  // used to not output 0 amplitudes to prevent two mirrored layers from overwriting each other
  public void mask(int val)
  {
	  if (val < 0)
		  val = 0;
	  if (val > 1)
		  val = 1;
	  
	  mask = val;
  }
 
}
